{******************************************************************************}
{                                MonthCalendarEx
{                                ---------------
{
{                       Copyright  2002 Pieter Zijlstra
{
{ An extension to the existing TMonthCalendar. It has an OnChange event
{ which is called when the user changes the Data by changing the month, year
{ or just the day itself.
{
{ E-mail: p.zylstra@hccnet.nl
{ Website: http://home.hccnet.nl/p.zylstra/
{===============================================================================
{                           This software is FREEWARE
{                           -------------------------
{
{ You may freely use it in any software, including commercial software, but
{ be-aware that the code is provided as-is, with no implied warranty.
{===============================================================================
{
{ Notes:
{ - Delphi version(s) 5, 6 & 7
{
{ Version history:
{ 1.00 30 jun 2002 - First public release.
{ 1.01 26 dec 2002 - Checked for D7, no changes.
{ 1.02 03 feb 2003 - Added e-mail and website in comments.
{******************************************************************************}
unit MonthCalendarEx;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls,
  CommCtrl, ComCtrls;

type
  TMonthCalendarEx = class(TMonthCalendar)
  private
    FSelectedDate: TDate;
    FOnChange: TNotifyEvent;
    procedure SetDate(Value: TDate);
    procedure SetSelectedDate(Value: TDate);
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
  protected
    function MsgSetDateTime(Value: TSystemTime): Boolean; override;
    procedure Change; dynamic;
  published
    constructor Create(AOwner: TComponent); override;
    property Date: TDate read FSelectedDate write SetDate;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('PZGoodies', [TMonthCalendarEx]);
end;

constructor TMonthCalendarEx.Create;
begin
  inherited;
  SetSelectedDate(inherited Date);
end;

procedure TMonthCalendarEx.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

function TMonthCalendarEx.MsgSetDateTime(Value: TSystemTime): Boolean;
begin
  Result := inherited MsgSetDateTime(Value);
  if Result then
    SetSelectedDate(SystemTimeToDateTime(Value));
end;

procedure TMonthCalendarEx.SetDate(Value: TDate);
begin
  inherited Date := Value;
end;

procedure TMonthCalendarEx.SetSelectedDate(Value: TDate);
begin
  Value := Trunc(Value);
  if FSelectedDate <> Value then
  begin
    FSelectedDate := Value;
    Change;
  end;
end;

function IsBlankSysTime(const ST: TSystemTime): Boolean;
type
  TFast = array[0..3] of DWORD;
begin
  Result := (TFast(ST)[0] or TFast(ST)[1] or TFast(ST)[2] or TFast(ST)[3]) = 0;
end;

procedure TMonthCalendarEx.CNNotify(var Message: TWMNotify);
var
  ST: PSystemTime;
begin
  inherited;
  with Message, NMHdr^ do
  begin
    case code of
      MCN_SELECT, MCN_SELCHANGE:
        begin
          ST := @PNMSelChange(NMHdr).stSelStart;
          if not IsBlankSysTime(ST^) then
          begin
            SetSelectedDate(SystemTimeToDateTime(ST^));
          end;
        end;
    end;
  end;
end;

end.
 